def include(declaration):
    """ Include another YAML file """
    expect_names = ['package', 'filename', 'override']
    declaration.expect(dict, names=expect_names)
    package = declaration.string('package')
    if package is not None:
        package = declaration.resolve(package)
    else:
        package = declaration.context.current_package()
    filename = declaration.string('filename', 'configure.yml')
    override = declaration.string('override',
                                  declaration.context.current_override())
    declaration.context.load(filename, package, override)



import unittest

class TestInclude(unittest.TestCase):
    def _callFUT(self, declaration):
        from repoze.configuration.directives import include
        return include(declaration)

    def test_bad_structure(self):
        declaration = DummyDeclaration(badstructure=True)
        self.assertRaises(ValueError, self._callFUT, declaration)

    def test_expect_names(self):
        declaration = DummyDeclaration(diff=True)
        self.assertRaises(ValueError, self._callFUT, declaration)
        
    def test_package_not_none_filename_none(self):
        structure = {'package':'here'}
        declaration = DummyDeclaration(structure=structure)
        self._callFUT(declaration)
        self.assertEqual(declaration.context.loaded,
                         ('configure.yml', 'here', False))

    def test_package_not_none_filename_not_none(self):
        structure = {'package':'here', 'filename':'here.yml'}
        declaration = DummyDeclaration(structure=structure)
        self._callFUT(declaration)
        self.assertEqual(declaration.context.loaded,
                         ('here.yml', 'here', False))

    def test_package_none_filename_none(self):
        structure = {}
        declaration = DummyDeclaration(structure=structure)
        self._callFUT(declaration)
        self.assertEqual(
            declaration.context.loaded, ('configure.yml', 'package', False))

    def test_package_none_filename_not_none(self):
        structure = {'filename':'here.yml'}
        declaration = DummyDeclaration(structure=structure)
        self._callFUT(declaration)
        self.assertEqual(declaration.context.loaded,
                         ('here.yml', 'package', False))

    def test_withoverride(self):
        structure = {'filename':'here.yml', 'override':True}
        declaration = DummyDeclaration(structure=structure)
        self._callFUT(declaration)
        self.assertEqual(declaration.context.loaded,
                         ('here.yml', 'package', True))

class DummyDeclaration:
    def __init__(self, **kw):
        self.diff = kw.get('diff', False)
        self.structure = kw.get('structure', {})
        self.badstructure = kw.get('badstructure', False)
        self.context = DummyContext()
        self.__dict__.update(kw)

    def expect(self, typ, names=()):
        if self.badstructure:
            raise ValueError
        if self.diff:
            raise ValueError

    def getvalue(self, name, default=None, pop=False):
        return self.structure.get(name, default)

    string = getvalue

    def resolve(self, dottedname):
        return dottedname

    def error(self, node, msg):
        raise ValueError(node, msg)
    
class DummyContext:
    loaded = None
    def current_package(self):
        return 'package'

    def current_override(self):
        return False

    def load(self, filename, package, override):
        self.loaded = (filename, package, override)




from yaml import SafeLoader
from pkg_resources import iter_entry_points

from repoze.configuration.exceptions import ConfigurationError
from repoze.configuration.declaration import YAMLDeclaration
from repoze.configuration.declaration import lineinfo

_marker = object()

class YAMLPluginLoader(SafeLoader):
    EP_GROUP = 'repoze.configuration.directive'
    def __init__(self, context, stream, iter_entry_points=iter_entry_points):
        self.context = context
        for point in list(iter_entry_points(self.EP_GROUP)):
            try:
                directive = point.load()
                self.add_constructor('!'+point.name, wrap_directive(directive))
            except ImportError:
                pass
        self.add_constructor('tag:yaml.org,2002:str', self.interpolate_str)
        SafeLoader.__init__(self, stream)
        while self.check_data():
            self.get_data()

    def interpolate_str(self, loader, node):
        value = loader.construct_scalar(node)
        try:
            value = self.context.interpolate(value)
        except KeyError, why:
            msg = 'Cannot interpolate %%(%s)s\n' % why[0]
            msg = msg + lineinfo(node)
            raise ConfigurationError(msg)
        return value

def wrap_directive(directive):
    def wrapper(loader, node):
        context = loader.context
        declaration = YAMLDeclaration(context, loader, node)
        directive(declaration)
    wrapper.wrapped = directive
    return wrapper




import inspect
from repoze.configuration.exceptions import ConfigurationError

_marker = object()

class Declaration(object):
    """ Abstract base class for declaration objects: __init__ only
    supplied for unit tests."""
    def __init__(self, **kw):
        self.__dict__.update(kw)
    
    @property
    def registry(self):
        return self.context.registry

    def expect(self, typ, names=None):
        """ Raise a ConfigurationError if:

        - The type of the declaration structure is not of the same
          type as the object passed as 'typ'.

        - ``names`` is not None and the names provided in the
          ``structure`` don't match the sequence of names supplied as
          ``names``.  This only works when the structure is a mapping
          type.

        Otherwise return None.
        """
        if not isinstance(self.structure, typ):
            self.error(
                'Bad structure for directive (%s instead of %s)' %
                (type(self.structure), typ))

        if names is not None:
            names_provided = set(self.structure.keys())
            names_expected = set(names)
            diff = names_provided.difference(names)

            if diff:
                self.error('Unknown key(s) in directive: %r' % diff)

    def getvalue(self, name, default=None, types=(), pop=False):
        """
        Return the value corresponding to ``name`` in the structure of
        this declaration.  If the ``types`` value is not empty, and
        the value found is not if a type listed in ``types``, raise a
        configuration error.  If the value does not exist, return the
        default.  If ``pop`` is true, pop the value from the structure
        before returning it.  Return the value.  This method only
        works if the declaration's structure is a mapping type.
        """
        if pop:
            value = self.structure.pop(name, default)
        else:
            value = self.structure.get(name, default)
        if value is default:
            return default
        if types:
            if not isinstance(value, types):
                self.error(
                    '"%s" attribute type is not one of the types %s: %r' % (
                    name, types, value))
        return value

    def boolean(self, name, default=False, pop=False):
        """ Return a boolean value based on the ``name`` passed.  This
        method only works if the declaration's structure is a mapping
        type.

        If ``name`` does not exist in the structure, return ``default``.

        If ``name`` exists in the structure, and is a string or
        unicode type, return a boolean True if the string is any of 
        ``('t', 'true', 'yes', 'on', '1')``, otherwise return False.

        If the ``name`` exists in the structure and is not a string or
        unicode type, return Python's bool(value).  If ``pop`` is
        true, pop the value from the structure before returning it."""
        val = self.getvalue(name, default, pop=pop)
        if isinstance(val, basestring):
            if val.lower() in ('t', 'true', 'yes', 'on', '1'):
                return True
            return False
        return bool(val)

    def string(self, name, default=None, pop=False):
        """
        Return the value corresponding to ``name`` in the structure of
        this declaration.  If the value exists and its type is not
        ``basestring``, raise a configuration error.  If the value
        does not exist, return the default.  If ``pop`` is true, pop
        the value from the structure before returning it.  This method
        only works if the declaration's structure is a mapping type.
        """
        return self.getvalue(name, default, types=(basestring,), pop=pop)

    def integer(self, name, default=None, pop=False):
        """
        Return the value corresponding to ``name`` in the structure of
        this declaration.  If the value exists and it cannot be
        converted to a Python integer, raise a configuration error.
        If the value does not exist, return the default.  If ``pop``
        is true, pop the value from the structure before returning it.
        This method only works if the declaration's structure is a
        mapping type.
        """
        value = self.getvalue(name, default, types=(basestring, int), pop=pop)
        if value is default:
            return default
        try:
            return int(value)
        except:
            self.error(
                '%s with value %s cannot be converted to an integer' %
                (name, value))

    def resolve(self, dottedname):
        """ Resolve a setuptools-style dotted name
        (e.g. ``module1.module2:moduelevel_attr``) to a Python object.
        If the name cannot be resolved, a ``ConfigurationError`` is
        raised."""
        try:
            return self.context.resolve(dottedname)
        except ImportError, why:
            self.error(
                str(why) + ' for dotted name %s' % dottedname)

    def error(self, msg):
        """ Raise a ``ConfigurationError`` with the message ``msg`` """
        msg = '%s\n%s' % (msg , self.lineinfo)
        raise ConfigurationError(msg)

    def call_later(self, func, *arg, **kw):
        """ Return a callback accepting no arguments which calls
        ``func`` (any callable) with *arg, **kw when called. """
        def callback():
            return func(*arg, **kw)
        return callback

    def action(self, callback, discriminator=None, override=False):
        """ Insert a deferred processing action.  ``callback`` is a
        callback that accepts no arguments.  ``discriminator`` is any
        hashable value which makes the action unique amongst all other
        actions during processing.  ``override``, if True, means that
        this action should override previous actions during
        processing."""
        return self.context.action(self, callback,
                                   discriminator=discriminator,
                                   override=override)

class YAMLDeclaration(Declaration):
    """
    Instances of this kind of object are passed to a directive.  A
    declaration represents the data parsed from a configuration file.
    It also has helper methods made available for use in directives.

    A declaration always has the following attributes:
    
    - structure: the Python data structure representing the parsed
      configuration file content.

    - lineinfo: a string representing line (column/row) information for the
      parsed directive content

    - context: a ``repoze.configuration.context.Context`` object
      representing 'global` state for this configuration session
    """
    def __init__(self, context, loader, node):
        self.context = context # required by superclass
        self._loader = loader
        self._node = node
        self._structure = _marker

    @property
    def lineinfo(self):
        return lineinfo(self._node)

    def get_structure(self):
        if self._structure is _marker:
            loader = self._loader
            structure_ctor = getattr(loader, 'construct_%s' % self._node.id)
            # scalar structures don't support the 'deep' argument, required
            # for mappings XXX there has to be a better way.
            if 'deep' in inspect.getargspec(structure_ctor)[0]:
                structure = structure_ctor(self._node, deep=True)
            else:
                structure = structure_ctor(self._node)
            self._structure = structure
        return self._structure

    def set_structure(self, structure):
        self._structure = structure

    structure = property(get_structure, set_structure)

def lineinfo(node):
    start_mark = node.start_mark
    end_mark = node.end_mark
    filename = start_mark.name
    try:
        f = open(filename, 'r')
        f.seek(start_mark.index)
        data = f.read(end_mark.index - start_mark.index) + 'in '
    except (OSError, IOError):
        data = ''
    msg = ('%slines %s:%s-%s:%s of file "%s"' % (
        data,
        start_mark.line,
        start_mark.column,
        end_mark.line,
        end_mark.column,
        start_mark.name,
        )
    )
    return msg



from repoze.configuration.exceptions import ConfigurationError # API
from repoze.configuration.exceptions import ConfigurationConflict # API
from repoze.configuration.context import Context # API

def load(filename='configure.yml', package=None, context=None, loader=None):
    """
    You can load configuration without executing it (without calling
    any callbacks) by using the ``load`` function.  ``load`` accepts a
    filename argument and a package argument.  The ``package``
    argument is optional.  If it is not specified, the filename is
    found in the current working directory.

    .. code-block:: python
       :linenos:

       >>> # load configuration without a package via an absolute path
       >>> from repoze.configuration import load
       >>> context = load('/path/to/configure.yml')
       >>> registry = context.registry

    After using ``load`` you can subsequently execute the directive
    actions using the ``execute()`` method of the returned context
    object.  Using ``repoze.configuration.load``, then an immediately
    subsequent ``context.execute()`` is exactly equivalent to calling
    ``repoze.configuration.execute``.

    See the ``execute`` documentation for the meanings of the
    arguments passed to this function.
    """
    if context is None:
        registry = {}
        context = Context(registry, loader)
    context.load(filename, package)
    return context

def execute(filename='configure.yml', package=None, context=None, loader=None):
    """
    ``execute`` loads the configuration, executes the actions
    implied by the configuration, and returns a context.  After
    successful execution, you can access the populated registry
    dictionary by referring to the context's ``registry`` attribute:

    ``execute`` accepts a ``filename`` argument and a ``package``
    argument.  The ``package`` argument is optional.  If it is not
    specified, the filename is found in the current working directory.

    For example:

    .. code-block:: python
       :linenos:

       >>> # load configuration without a package via an absolute path
       >>> from repoze.configuration import execute
       >>> context = execute('/path/to/configure.yml')
       >>> registry = context.registry

       >>> # load configuration from the 'configure.yml' file within 'somepackage'
       >>> from repoze.configuration import load
       >>> import somepackage
       >>> context = execute('configure.yml', package=somepackge)
       >>> registry = context.registry
    """
    context = load(filename, package, context, loader)
    context.execute()
    return context

    
        


    





import unittest

class TestDeclaration(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration.declaration import Declaration
        return Declaration

    def _makeOne(self, **kw):
        return self._getTargetClass()(**kw)

    def test_registry(self):
        context = DummyContext('registry')
        decl = self._makeOne(context = context)
        self.assertEqual(decl.registry, 'registry')

    def test_expect_no_names_wrongtype(self):
        from repoze.configuration.exceptions import ConfigurationError
        decl = self._makeOne(structure=[], lineinfo='')
        self.assertRaises(ConfigurationError, decl.expect, dict)

    def test_expect_no_names_righttype(self):
        decl = self._makeOne(structure={}, lineinfo='')
        decl.expect(dict) # doesnt blow up

    def test_expect_names_wrong(self):
        from repoze.configuration.exceptions import ConfigurationError
        decl = self._makeOne(structure={'foo':'bar'}, lineinfo='')
        self.assertRaises(ConfigurationError, decl.expect, dict, ['a'])

    def test_expect_names_right(self):
        decl = self._makeOne(structure={'foo':'bar'}, lineinfo='')
        decl.expect(dict, ['foo', 'bar']) # doesnt blow up

    def test_getvalue_wrongtype(self):
        from repoze.configuration.exceptions import ConfigurationError
        decl = self._makeOne(structure = {'a':1}, lineinfo='')
        self.assertRaises(ConfigurationError, decl.getvalue, 'a', types=(str,))
        
    def test_getvalue_default(self):
        decl = self._makeOne(structure={})
        self.assertEqual(decl.getvalue('a'), None)
        
    def test_getvalue_get(self):
        structure = {'a':'1'}
        decl = self._makeOne(structure=structure)
        self.assertEqual(decl.getvalue('a'), '1')
        self.assertEqual(structure, {'a':'1'})

    def test_getvalue_pop(self):
        structure = {'a':1}
        decl = self._makeOne(structure=structure)
        val = decl.getvalue('a', pop=True)
        self.assertEqual(val, 1)
        self.assertEqual(structure, {})
        
    def test_boolean_true(self):
        for val in ('t', 'true', 'yes', 'on', '1'):
            decl = self._makeOne(structure = {'a':val})
            val = decl.boolean('a')
            self.assertEqual(val, True)

    def test_boolean_false(self):
        decl = self._makeOne(structure = {'a':'false'})
        val = decl.boolean('a')
        self.assertEqual(val, False)

    def test_boolean_unknown(self):
        decl = self._makeOne(structure = {'a':None})
        val = decl.boolean('a')
        self.assertEqual(val, False)

    def test_boolean_pop(self):
        structure = {'a':'false'}
        decl = self._makeOne(structure=structure)
        val = decl.boolean('a', pop=True)
        self.assertEqual(val, False)
        self.assertEqual(structure, {})

    def test_boolean_get(self):
        structure = {'a':'false'}
        decl = self._makeOne(structure=structure)
        val = decl.boolean('a')
        self.assertEqual(val, False)
        self.assertEqual(structure, {'a':'false'})

    def test_string_get(self):
        structure = {'a':'yo!'}
        decl = self._makeOne(structure=structure)
        val = decl.string('a')
        self.assertEqual(val, 'yo!')
        self.assertEqual(structure, {'a':'yo!'})
        
    def test_string_wrongtype(self):
        from repoze.configuration.exceptions import ConfigurationError
        decl = self._makeOne(structure = {'a':1}, lineinfo='')
        self.assertRaises(ConfigurationError, decl.string, 'a')

    def test_string_pop(self):
        structure = {'a':'yo!'}
        decl = self._makeOne(structure=structure)
        val = decl.string('a', pop=True)
        self.assertEqual(val, 'yo!')
        self.assertEqual(structure, {})

    def test_integer_get(self):
        structure = {'a':1}
        decl = self._makeOne(structure=structure)
        val = decl.integer('a')
        self.assertEqual(val, 1)
        self.assertEqual(structure, {'a':1})
        
    def test_integer_pop(self):
        structure = {'a':1}
        decl = self._makeOne(structure=structure)
        val = decl.integer('a', pop=True)
        self.assertEqual(val, 1)
        self.assertEqual(structure, {})

    def test_integer_default(self):
        structure = {}
        decl = self._makeOne(structure=structure)
        val = decl.integer('a')
        self.assertEqual(val, None)

    def test_intger_fromstring(self):
        decl = self._makeOne(structure = {'a':'1'})
        val = decl.integer('a')
        self.assertEqual(val, 1)

    def test_intger_fromstring_error(self):
        from repoze.configuration.exceptions import ConfigurationError
        decl = self._makeOne(structure = {'a':'flub'}, lineinfo='')
        self.assertRaises(ConfigurationError, decl.integer, 'a')

    def test_resolve(self):
        context = DummyContext({})
        decl = self._makeOne(context=context)
        result = decl.resolve('name')
        self.assertEqual(result, 'resolved')

    def test_resolve_importerror(self):
        from repoze.configuration.exceptions import ConfigurationError
        context = DummyContext({}, resolve_err=True)
        decl = self._makeOne(context=context, lineinfo='')
        self.assertRaises(ConfigurationError, decl.resolve, 'name')

    def test_error(self):
        from repoze.configuration.exceptions import ConfigurationError
        decl = self._makeOne(lineinfo='lineinfo')
        self.assertRaises(ConfigurationError, decl.error, 'message')

    def test_call_later(self):
        decl = self._makeOne()
        class Callback:
            def __call__(self, *arg, **kw):
                self.arg = arg
                self.kw = kw
                return True
        callback = Callback()
        sequence = [1,2,3]
        deferred = decl.call_later(callback, 1, 2, name=1, *sequence)
        result = deferred()
        self.assertEqual(result, True)
        self.assertEqual(callback.arg, (1,2,1,2,3))
        self.assertEqual(callback.kw, {'name':1})

    def test_action(self):
        context = DummyContext({})
        decl = self._makeOne(context=context)
        decl.action('callback', discriminator='discriminator',
                    override='override')
        self.assertEqual(context.actions,
                         [(decl, 'callback', 'discriminator', 'override')])

class TestYAMLDeclaration(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration.declaration import YAMLDeclaration
        return YAMLDeclaration

    def _makeOne(self, context, loader, node):
        return self._getTargetClass()(context, loader, node)

    def test_lineinfo(self):
        context = None
        loader = None
        node = DummyNode()
        decl = self._makeOne(context, loader, node)
        line = decl.lineinfo
        self.assertEqual(line, 'lines 1:1-1:1 of file "dummy"')

    def test_get_structure_deep(self):
        context = DummyContext()
        loader = DummyLoader(context)
        node = DummyNode(id='deep')
        decl = self._makeOne(context, loader, node)
        result = decl.structure
        self.assertEqual(result, 'deep')
        self.assertEqual(decl._structure, 'deep')

    def test_get_structure_shallow(self):
        context = DummyContext()
        loader = DummyLoader(context)
        node = DummyNode(id='shallow')
        decl = self._makeOne(context, loader, node)
        result = decl.structure
        self.assertEqual(result, 'shallow')
        self.assertEqual(decl._structure, 'shallow')

    def test_set_structure(self):
        context = DummyContext()
        loader = DummyLoader(context)
        node = DummyNode(id='shallow')
        decl = self._makeOne(context, loader, node)
        decl.structure = 'structure'
        self.assertEqual(decl._structure, 'structure')

class Test_lineinfo(unittest.TestCase):
    def _callFUT(self, node):
        from repoze.configuration.declaration import lineinfo
        return lineinfo(node)
    
    def test_withfile(self):
        import os
        here = os.path.normpath(os.path.dirname(__file__))
        fixtures = os.path.join(here, 'fixtures')
        filename = os.path.join(fixtures, 'conflict1.yml')

        node = DummyNode()

        node.start_mark.line = 1
        node.start_mark.column = 2
        node.start_mark.index = 0
        node.start_mark.name = filename
        
        node.end_mark.line = 1
        node.end_mark.column = 2
        node.end_mark.index = 0
        node.end_mark.index = 50
        node.end_mark.name = filename

        msg = self._callFUT(node)

        lines = msg.split('\n')
        self.assertEqual(len(lines), 3)
        self.assertEqual(lines[0], '--- !abc')
        self.assertEqual(lines[1], 'foo: 1')
        self.assertEqual(lines[2], 'in lines 1:2-1:2 of file "%s"' % filename)

    def test_file_exception(self):
        node = DummyNode()
        msg = self._callFUT(node)

        lines = msg.split('\n')
        self.assertEqual(len(lines), 1)
        self.assertEqual(lines[0], 'lines 1:1-1:1 of file "dummy"')

## class DummyDirective:
##     def __init__(self, result, raise_exc=False):
##         self.result = result
##         self.raise_exc = raise_exc
        
##     def __call__(self, declaration):
##         if self.raise_exc:
##             raise KeyError('yo')
##         return self.result

##     def action(self, callback, discriminator=None, override=False):
##         self.actions.append((callback, discriminator, override))

##     def test_directive_calls_action(self):
##         directive = DummyDirective([('a', 'b')])
##         constructor = self._callFUT(directive)
##         node = DummyNode()
##         context = DummyContext()
##         loader = DummyLoader(context)
##         constructor(loader, node)
##         self.assertEqual(context.actions, [(('a', 'b'), node)] )

##     def test_directive_returns_dict(self):
##         directive = DummyDirective({'a':'1'})
##         constructor = self._callFUT(directive)
##         node = DummyNode()
##         context = DummyContext()
##         loader = DummyLoader(context)
##         result = constructor(loader, node)
##         self.assertEqual(context.actions, [({'a':'1'}, node)] )

##     def test_withexception(self):
##         directive = DummyDirective([('a', 'b')], raise_exc=True)
##         constructor = self._callFUT(directive)
##         node = DummyNode()
##         context = DummyContext()
##         loader = DummyLoader(context)
##         self.assertRaises(KeyError, constructor, loader, node)

##     def test_construct_mapping_uses_deep(self):
##         directive = DummyDirective({'a':1})
##         constructor = self._callFUT(directive)
##         node = DummyNode('mapping')
##         context = DummyContext()
##         loader = DummyLoader(context)
##         constructor(loader, node)
##         self.assertEqual(context.actions, [({'a':1}, node)] )
##         self.assertEqual(loader.deep, True)

class DummyNode:
    def __init__(self, id='theid'):
        self.id = id
        self.start_mark = DummyMark()
        self.end_mark = DummyMark()
        
class DummyMark:
    line = 1
    column = 1
    name = 'dummy'

class DummyContext:
    def __init__(self, registry=None, resolve_err=False):
        if registry is None:
            registry = {}
        self.registry = registry
        self.actions = []
        self.resolve_err = resolve_err

    def resolve(self, name):
        if self.resolve_err:
            raise ImportError('whatever')
        return 'resolved'

    def action(self, directive, callback, discriminator=None, override=False):
        self.actions.append((directive, callback, discriminator, override))
        
        
class DummyLoader:
    def __init__(self, context):
        self.context = context

    def construct_deep(self, node, deep=True):
        return 'deep'

    def construct_shallow(self, node):
        return 'shallow'
    
    



import os
import pkg_resources
import re

from repoze.configuration.exceptions import ConfigurationConflict
from repoze.configuration.loader import YAMLPluginLoader

_INTERP = re.compile(r"%\(([^)]*)\)s")

class Context(object):

    def __init__(self, registry, loader=None):
        if loader is None:
            loader = YAMLPluginLoader
        self.loader = loader
        self.registry = registry
        self.actions = []
        self.stack = []
        self.discriminators = {}

    def interpolate(self, value):
        def _interpolation_replace(match):
            s = match.group(1)
            if not s in self.registry:
                raise KeyError(s)
            return self.registry[s]
        if '%(' in value:
            value = _INTERP.sub(_interpolation_replace, value)
        return value.encode('utf-8')

    def action(self, declaration, callback, discriminator=None, override=False):
        stack_override = self.stack and self.stack[-1]['override']
        effective_override = override or stack_override
        if not effective_override:
            if discriminator in self.discriminators:
                conflicting_action = self.discriminators[discriminator]
                raise ConfigurationConflict(declaration,
                                            conflicting_action.declaration)

        action = Action(discriminator, callback, declaration)
        self.actions.append(action)
        self.discriminators[discriminator] = action

    def resolve(self, dottedname):
        if dottedname.startswith('.') or dottedname.startswith(':'):
            package = self.current_package()
            if not package:
                raise ImportError('name "%s" is irresolveable (no package)' %
                    dottedname)
            if dottedname in ['.', ':']:
                dottedname = package.__name__
            else:
                dottedname = package.__name__ + dottedname
        return pkg_resources.EntryPoint.parse(
            'x=%s' % dottedname).load(False)

    def current_package(self):
        if not self.stack:
            return None
        return self.stack[-1]['package']

    def current_override(self):
        if not self.stack:
            return False
        return self.stack[-1]['override']

    def stream(self, filename, package=None):
        if os.path.isabs(filename):
            return open(filename)
        if package is None:
            package = self.current_package()
        if package is None:
            return open(filename)
        else:
            return pkg_resources.resource_stream(package.__name__, filename)
        
    def load(self, filename, package, override=False, loader=None):
        stream = self.stream(filename, package)
        self.stack.append({'filename':filename, 'package':package,
                           'override':override})
        if loader is None:
            loader = self.loader
        try:
            loader(self, stream)
        finally:
            self.stack.pop()

    def execute(self):
        for action in self.actions:
            action.execute()
        return self.registry

class Action(object):
    def __init__(self, discriminator, callback, declaration):
        self.discriminator = discriminator
        self.callback = callback
        self.declaration = declaration
        
    def execute(self):
        if self.callback is not None:
            self.callback()




import unittest

class TestContext(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration import Context
        return Context

    def _makeOne(self, registry=None, loader=None):
        if registry is None:
            registry = {}
        return self._getTargetClass()(registry, loader)

    def test_ctor(self):
        registry = {}
        context = self._makeOne(registry, 'loader')
        self.assertEqual(context.registry, registry)
        self.assertEqual(context.loader, 'loader')
        self.assertEqual(context.stack, [])
        self.assertEqual(context.actions, [])

    def test_interpolate(self):
        registry = {'here':'/here', 'there':'/there'}
        context = self._makeOne(registry)
        result = context.interpolate('Here is %(here)s, there is %(there)s')
        self.assertEqual(result, 'Here is /here, there is /there')

    def test_interpolate_nothing_to_interpolate(self):
        registry = {'here':'/here', 'there':'/there'}
        context = self._makeOne(registry)
        result = context.interpolate('Here is %(here')
        self.assertEqual(result, 'Here is %(here')

    def test_interpolate_keyerror(self):
        registry = {}
        context = self._makeOne(registry)
        self.assertRaises(KeyError, context.interpolate, 'Here is %(here)s')

    def test_action(self):
        context = self._makeOne()
        context.action('declaration', 'callback', discriminator='discriminator')
        self.assertEqual(len(context.actions), 1)
        action = context.actions[0]
        self.assertEqual(action.discriminator, 'discriminator')
        self.assertEqual(action.callback, 'callback')
        self.assertEqual(action.declaration, 'declaration')
        self.assertEqual(context.discriminators['discriminator'],
                         context.actions[0])

    def test_action_withconflict(self):
        from repoze.configuration.context import ConfigurationConflict
        context = self._makeOne()
        context.discriminators['discriminator'] = DummyAction()
        context.stack = [{'override':False}]
        declaration = DummyDeclaration()
        self.assertRaises(ConfigurationConflict,
                          context.action,
                          declaration, 'callback', 'discriminator')

    def test_action_withconflict_local_override_no_stack_override(self):
        context = self._makeOne()
        context.discriminators['discriminator'] = DummyAction()
        context.stack = [{'override':False}]
        context.action('declaration', 'callback', discriminator='discriminator',
                       override=True)
        self.assertEqual(len(context.actions), 1)
        action = context.actions[0]
        self.assertEqual(action.discriminator, 'discriminator')
        self.assertEqual(action.callback, 'callback')
        self.assertEqual(action.declaration, 'declaration')
        self.assertEqual(context.discriminators['discriminator'],
                         context.actions[0])

    def test_action_withconflict_stack_override_no_local_override(self):
        context = self._makeOne()
        context.discriminators['discriminator'] = DummyAction()
        context.stack = [{'override':True}]
        context.action('declaration', 'callback', discriminator='discriminator',
                       override=False)
        self.assertEqual(len(context.actions), 1)
        action = context.actions[0]
        self.assertEqual(action.discriminator, 'discriminator')
        self.assertEqual(action.callback, 'callback')
        self.assertEqual(action.declaration, 'declaration')
        self.assertEqual(context.discriminators['discriminator'],
                         context.actions[0])

    def test_resolve_absolute(self):
        from repoze.configuration.tests.fixtures import fixturefunc
        context = self._makeOne()
        result = context.resolve(
            'repoze.configuration.tests.fixtures:fixturefunc')
        self.assertEqual(result, fixturefunc)
        
    def test_irrresolveable_absolute(self):
        context = self._makeOne()
        self.assertRaises(ImportError, context.resolve,
            'repoze.configuration.tests.fixtures:nonexisting')

    def test_resolve_relative_startswith_colon(self):
        from repoze.configuration.tests import fixtures
        from repoze.configuration.tests.fixtures import fixturefunc
        context = self._makeOne()
        context.stack.append({'package':fixtures})
        result = context.resolve(':fixturefunc')
        self.assertEqual(result, fixturefunc)

    def test_resolve_relative_startswith_dot(self):
        from repoze.configuration import tests
        from repoze.configuration.tests.fixtures import fixturefunc
        context = self._makeOne()
        context.stack.append({'package':tests})
        result = context.resolve('.fixtures:fixturefunc')
        self.assertEqual(result, fixturefunc)

    def test_resolve_relative_is_dot(self):
        from repoze.configuration import tests
        context = self._makeOne()
        context.stack.append({'package':tests})
        result = context.resolve('.')
        self.assertEqual(result, tests)
        
    def test_resolve_relative_nocurrentpackage(self):
        context = self._makeOne()
        self.assertRaises(ImportError, context.resolve, '.fixturefunc')

    def test_irrresolveable_relative(self):
        from repoze.configuration.tests import fixtures
        context = self._makeOne()
        context.stack.append({'package':fixtures})
        self.assertRaises(ImportError, context.resolve, ':notexisting')

    def test_current_package_nostack(self):
        context = self._makeOne()
        self.assertEqual(context.current_package(), None)

    def test_current_package_withstack(self):
        context = self._makeOne()
        context.stack.append({'package':'abc'})
        self.assertEqual(context.current_package(), 'abc')

    def test_current_override_nostack(self):
        context = self._makeOne()
        self.assertEqual(context.current_override(), False)

    def test_current_override_withstack(self):
        context = self._makeOne()
        context.stack.append({'override':True})
        self.assertEqual(context.current_override(), True)
        
    def test_stream_absolute(self):
        import os
        from repoze.configuration.tests import fixtures
        filename = os.path.join(os.path.dirname(
            os.path.abspath(fixtures.__file__)), '__init__.py')
        context = self._makeOne()
        stream = context.stream(filename)
        self.assertEqual(stream.read(), open(filename).read())

    def test_stream_nopackage_nocurrentpackage(self):
        import os
        from repoze.configuration.tests import fixtures
        old_cwd = os.getcwd()
        fixtures = os.path.dirname(os.path.abspath(fixtures.__file__))
        filename = os.path.join(fixtures, '__init__.py')
        try:
            os.chdir(fixtures)
            context = self._makeOne()
            stream = context.stream('__init__.py')
            self.assertEqual(stream.read(), open(filename).read())
        finally:
            os.chdir(old_cwd)
        
    def test_stream_nopackage_currentpackage(self):
        import os
        from repoze.configuration.tests import fixtures
        context = self._makeOne()
        context.stack.append({'package':fixtures})
        fixtures = os.path.dirname(os.path.abspath(fixtures.__file__))
        filename = os.path.join(fixtures, '__init__.py')
        expected = open(filename).read()
        self.assertEqual(context.stream('__init__.py').read(), expected)

    def test_load_standard_loader(self):
        def loader(context, stream):
            context.loaded = True
        context = self._makeOne(loader=loader)
        from repoze.configuration.tests import fixtures
        context.load('configure.yml', fixtures)
        self.assertEqual(context.stack, [])
        self.assertEqual(context.loaded, True)

    def test_load_custom_loader(self):
        def loader(context, stream):
            context.loaded = True
        context = self._makeOne()
        from repoze.configuration.tests import fixtures
        context.load('configure.yml', fixtures, loader=loader)
        self.assertEqual(context.stack, [])
        self.assertEqual(context.loaded, True)
                                          
        # doesn't blow up
        

    def test_execute(self):
        registry = {}
        context = self._makeOne(registry)
        actions = [ DummyAction(), DummyAction()]
        context.actions = actions
        result = context.execute()
        self.assertEqual([action.executed for action in actions], [True, True])
        self.failUnless(result is registry)

class TestAction(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration.context import Action
        return Action

    def _makeOne(self, discriminator, callback, node):
        return self._getTargetClass()(discriminator, callback, node)

    def test_ctor(self):
        action = self._makeOne('discriminator', 'callback', 'declaration')
        self.assertEqual(action.discriminator, 'discriminator')
        self.assertEqual(action.callback, 'callback')
        self.assertEqual(action.declaration, 'declaration')

    def test_execute(self):
        class Callback:
            def __call__(self):
                self.called = True
        callback = Callback()
        action = self._makeOne('discriminator', callback, 'declaration')
        action.execute()
        self.assertEqual(callback.called, True)

    def test_execute_exception(self):
        class Callback:
            def __call__(self):
                raise ValueError('foo')
        callback = Callback()
        declaration = DummyDeclaration()
        action = self._makeOne('discriminator', callback, declaration)
        self.assertRaises(ValueError, action.execute)

class DummyAction:
    executed = False
    def __init__(self):
        self.declaration = DummyDeclaration()

    def execute(self):
        self.executed = True

class DummyDeclaration:
    lineinfo = 'lineinfo'
    



import unittest

class TestConfigurationConflict(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration.exceptions import ConfigurationConflict
        return ConfigurationConflict

    def _makeOne(self, node1, node2):
        error = self._getTargetClass()(node1, node2)
        return error

    def test_ctor(self):
        declaration1 = DummyDeclaration()
        declaration2 = DummyDeclaration()
        error = self._makeOne(declaration1, declaration2)
        self.assertEqual(error.declaration1, declaration1)
        self.assertEqual(error.declaration2, declaration2)
        lines = error.msg.split('\n')
        self.assertEqual(len(lines), 7)
        self.assertEqual(lines[0], 'Conflicting declarations:')
        self.assertEqual(lines[1], '')
        self.assertEqual(lines[2], 'lineinfo')
        self.assertEqual(lines[3], '')
        self.assertEqual(lines[4], 'conflicts with')
        self.assertEqual(lines[5], '')
        self.assertEqual(lines[6], 'lineinfo')


class DummyDeclaration(object):
    lineinfo = 'lineinfo'
    



class ConfigurationError(Exception):
    """ The exception type raised when a configuration file cannot be
    executed or loaded due to a configuration error """
    pass

class ConfigurationConflict(ConfigurationError):
    """ The exception type raised when a configuration contains
    conflicting declarations (the discriminators of two declarations
    conflict) """
    def __init__(self, declaration1, declaration2):
        self.declaration1 = declaration1
        self.declaration2 = declaration2
        self.msg = str(self)

    def __str__(self):
        message = []
        message.append('Conflicting declarations:')
        message.append(self.declaration2.lineinfo)
        message.append('conflicts with')
        message.append(self.declaration1.lineinfo)
        return '\n\n'.join(message)




import unittest

class TestYAMLPluginLoader(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration.loader import YAMLPluginLoader
        return YAMLPluginLoader

    def _makeOne(self, context, stream, iter):
        return self._getTargetClass()(context, stream, iter)

    def test_ctor_load_import_error(self):
        import os
        from repoze.configuration.tests import fixtures
        directory = os.path.dirname(os.path.abspath(fixtures.__file__))
        file = os.path.join(directory, 'configure.yml')
        def directive(context, structure):
            return 'success'
        point = DummyPoint(directive, raise_load_exc=True)
        def iter_entry_points(group, suffix=None):
            yield point
        context = DummyContext()
        loader = self._makeOne(context, open(file), iter_entry_points)
        self.assertEqual(loader.context, context)
        self.failIf('!point' in loader.yaml_constructors) # doesnt blow up

    def test_ctor_ok(self):
        import os
        from repoze.configuration.tests import fixtures
        directory = os.path.dirname(os.path.abspath(fixtures.__file__))
        file = os.path.join(directory, 'configure.yml')
        def directive(context, structure):
            return 'success'
        point = DummyPoint(directive)
        def iter_entry_points(group, suffix=None):
            yield point
        context = DummyContext()
        loader = self._makeOne(context, open(file), iter_entry_points)
        self.assertEqual(loader.context, context)
        self.assertEqual(loader.yaml_constructors['!point'].wrapped, directive)

    def test_interpolate_str(self):
        from yaml.nodes import ScalarNode
        import StringIO
        context = DummyContext()
        loader = DummyLoader(context)
        node = ScalarNode('foo', 'scalar')
        f = StringIO.StringIO()
        loader = self._makeOne(context, f, lambda *arg: ())
        self.assertEqual(loader.interpolate_str(loader, node), 'scalar')
        
    def test_interpolate_str_exc(self):
        from repoze.configuration.exceptions import ConfigurationError
        from yaml.nodes import ScalarNode
        import StringIO
        context = DummyContext(interpolation_exc=True)
        loader = DummyLoader(context)
        node = ScalarNode('foo', 'scalar', DummyMark(), DummyMark())
        f = StringIO.StringIO()
        loader = self._makeOne(context, f, lambda *arg: ())
        self.assertRaises(ConfigurationError,
                          loader.interpolate_str, loader, node)

class Test_wrap_directive(unittest.TestCase):
    def _callFUT(self, directive):
        from repoze.configuration.loader import wrap_directive
        return wrap_directive(directive)

    def test_it(self):
        directive = DummyDirective()
        wrapper = self._callFUT(directive)
        self.assertEqual(wrapper.wrapped, directive)
        loader = DummyLoader(None)
        wrapper(loader, None)
        self.assertEqual(directive.declaration.context, loader.context)
        self.assertEqual(directive.declaration._loader, loader)
        self.assertEqual(directive.declaration._node, None)

class DummyLoader:
    def __init__(self, context):
        self.context = context

    def construct_theid(self, node):
        return {}

    def construct_mapping(self, node, deep=False):
        self.deep = deep
        return {}

    def construct_scalar(self, node):
        return 'scalar'

class DummyContext:
    def __init__(self, interpolation_exc=False):
        self.actions = []
        self.interpolation_exc = interpolation_exc

    def interpolate(self, value):
        if self.interpolation_exc:
            raise KeyError(value)
        return value

    def load(self, filename, package=None, override=False):
        self.loaded = (filename, package, override)

    def current_package(self):
        return None

    def current_override(self):
        return False
        
class DummyPoint:
    name = 'point'
    def __init__(self, directive, raise_load_exc=False):
        self.directive = directive
        self.raise_load_exc = raise_load_exc

    def load(self):
        if self.raise_load_exc:
            raise ImportError('foo')
        return self.directive

class DummyMark:
    line = 1
    column = 1
    name = 'dummy'
    
class DummyDirective:
    def __call__(self, declaration):
        self.declaration = declaration
        
